@@ -43,7 +43,7 @@ def do_register_localstack_plugins():
         from localstack.services.logs import logs_listener, logs_starter
         from localstack.services.infra import (
             start_sns, start_route53, start_elasticsearch_service, start_lambda, start_sts, start_ssm,
-            start_redshift, start_firehose, start_cloudwatch, start_dynamodbstreams, start_acm
+            start_redshift, start_firehose, start_dynamodbstreams, start_acm
         )
         from localstack.services.events import events_listener, events_starter
         from localstack.services.plugins import Plugin, register_plugin
@@ -54,6 +54,7 @@ def do_register_localstack_plugins():
         from localstack.services.cloudformation import cloudformation_listener, cloudformation_starter
         from localstack.services.secretsmanager import secretsmanager_listener
         from localstack.services.secretsmanager import secretsmanager_starter
+        from localstack.services.cloudwatch import cloudwatch_listener, cloudwatch_starter
 
         register_plugin(Plugin(
             'edge',
@@ -76,7 +77,8 @@ def do_register_localstack_plugins():
 
         register_plugin(Plugin(
             'cloudwatch',
-            start=start_cloudwatch))
+            start=cloudwatch_starter.start_cloudwatch,
+            listener=cloudwatch_listener.UPDATE_CLOUD_WATCH))
 
         register_plugin(Plugin(
             'dynamodb',
