@@ -3,7 +3,7 @@ import logging
 from moto.apigateway import models as apigateway_models
 from moto.apigateway.models import Resource, Integration
 from moto.apigateway.responses import APIGatewayResponse
-from moto.apigateway.exceptions import MethodNotFoundException, NoIntegrationDefined
+from moto.apigateway.exceptions import NoIntegrationDefined
 from moto.apigateway.utils import create_id
 from localstack import config
 from localstack.utils.common import short_uid, to_str
@@ -17,14 +17,14 @@ def apply_patches():
         resource = self.get_resource(function_id, resource_id)
         method = resource.get_method(method_type)
         if not method:
-            raise MethodNotFoundException()
+            raise NoIntegrationDefined()
 
         return resource.resource_methods.pop(method_type)
 
     def apigateway_models_resource_get_method(self, method_type):
         method = self.resource_methods.get(method_type)
         if not method:
-            raise MethodNotFoundException()
+            raise NoIntegrationDefined()
 
         return method
 
@@ -117,6 +117,20 @@ def apply_patches():
 
         return 400, {}, ''
 
+    def apigateway_response_resource_methods(self, request, *args, **kwargs):
+        result = apigateway_response_resource_methods_orig(self, request, *args, **kwargs)
+        if len(result) != 3:
+            return result
+        authorization_type = self._get_param('authorizationType')
+        if authorization_type in ['CUSTOM', 'COGNITO_USER_POOLS']:
+            data = json.loads(result[2])
+            if not data.get('authorizerId'):
+                payload = json.loads(to_str(request.data))
+                if 'authorizerId' in payload:
+                    data['authorizerId'] = payload['authorizerId']
+                    result = result[0], result[1], json.dumps(data)
+        return result
+
     if not hasattr(apigateway_models.APIGatewayBackend, 'put_rest_api'):
         apigateway_response_restapis_individual_orig = APIGatewayResponse.restapis_individual
         APIGatewayResponse.restapis_individual = apigateway_response_restapis_individual
@@ -128,6 +142,8 @@ def apply_patches():
     apigateway_models.Resource.get_method = apigateway_models_resource_get_method
     apigateway_models.Resource.get_integration = apigateway_models_resource_get_integration
     apigateway_models.Resource.delete_integration = apigateway_models_resource_delete_integration
+    apigateway_response_resource_methods_orig = APIGatewayResponse.resource_methods
+    APIGatewayResponse.resource_methods = apigateway_response_resource_methods
 
 
 def start_apigateway(port=None, backend_port=None, asynchronous=None, update_listener=None):
