@@ -27,11 +27,10 @@ from localstack import config
 from localstack.constants import TEST_AWS_ACCOUNT_ID, MOTO_ACCOUNT_ID
 from localstack.utils.aws import aws_stack, aws_responses
 from localstack.utils.common import (
-    FuncThread, short_uid, recurse_object, clone, json_safe, md5, canonical_json, get_free_tcp_port, Mock, start_thread)
+    FuncThread, short_uid, recurse_object, clone, json_safe, md5, canonical_json,
+    get_free_tcp_port, Mock, start_thread, edge_ports_info)
 from localstack.stepfunctions import models as sfn_models
-from localstack.services.infra import (
-    get_service_protocol, start_proxy_for_service, do_run, canonicalize_api_names
-)
+from localstack.services.infra import start_proxy_for_service, do_run, canonicalize_api_names
 from localstack.utils.bootstrap import setup_logging
 from localstack.utils.cloudformation import template_deployer
 from localstack.services.cloudformation import service_models
@@ -58,6 +57,7 @@ MODEL_MAP = {
     'AWS::ApiGateway::Method': apigw_models.Method,
     'AWS::ApiGateway::Resource': apigw_models.Resource,
     'AWS::ApiGateway::RestApi': apigw_models.RestAPI,
+    'AWS::ApiGateway::Stage': apigw_models.Stage,
     'AWS::StepFunctions::StateMachine': sfn_models.StateMachine,
     'AWS::CloudFormation::Stack': service_models.CloudFormationStack,
     'AWS::SSM::Parameter': service_models.SSMParameter,
@@ -72,12 +72,11 @@ MODEL_MAP = {
 
 def start_cloudformation(port=None, asynchronous=False, update_listener=None):
     port = port or config.PORT_CLOUDFORMATION
-    print('Starting mock CloudFormation service in %s ports %s (recommended) and %s (deprecated)...' % (
-        get_service_protocol(), config.EDGE_PORT, port))
+    print('Starting mock CloudFormation service on %s ...' % edge_ports_info())
     backend_port = get_free_tcp_port()
     start_proxy_for_service('cloudformation', port, backend_port, update_listener)
     if RUN_SERVER_IN_PROCESS:
-        cmd = 'python "%s" cloudformation -p %s -H 0.0.0.0' % (__file__, backend_port)
+        cmd = '%s "%s" cloudformation -p %s -H 0.0.0.0' % (sys.executable, __file__, backend_port)
         env_vars = {'PYTHONPATH': ':'.join(sys.path)}
         return do_run(cmd, asynchronous, env_vars=env_vars)
     else:
@@ -109,8 +108,8 @@ def get_entity_id(entity, resource_json=None):
     for attr in attr_candidates:
         if hasattr(entity, attr):
             if attr in ['id', 'name'] and not isinstance(entity, types_with_ref_as_id_or_name):
-                LOG.warning('Unable to find ARN, using "%s" instead: %s - %s',
-                            attr, resource_json, entity)
+                LOG.warning('Unable to find ARN, using "%s" instead: %s - %s', attr, resource_json, entity)
+
             return getattr(entity, attr)
         if hasattr(entity, 'get_cfn_attribute'):
             try:
@@ -200,7 +199,7 @@ def update_dynamodb_index_resource(resource):
                 glob_index['ProvisionedThroughput'] = {'ReadCapacityUnits': 99, 'WriteCapacityUnits': 99}
 
 
-def add_default_resource_props(resource_props, stack_name, resource_name=None):
+def add_default_resource_props(resource_props, stack_name, resource_name=None, resource_id=None):
     """ Apply some fixes to resource props which otherwise cause deployments to fail """
 
     res_type = resource_props['Type']
@@ -218,8 +217,8 @@ def add_default_resource_props(resource_props, stack_name, resource_name=None):
     if res_type == 'AWS::SQS::Queue' and not props.get('QueueName'):
         props['QueueName'] = 'queue-%s' % short_uid()
 
-    if res_type == 'AWS::ApiGateway::RestApi':
-        props['Name'] = props.get('Name') or resource_name
+    if res_type == 'AWS::ApiGateway::RestApi' and not props.get('Name'):
+        props['Name'] = resource_name
 
     if res_type == 'AWS::DynamoDB::Table':
         update_dynamodb_index_resource(resource_props)
@@ -231,8 +230,10 @@ def add_default_resource_props(resource_props, stack_name, resource_name=None):
     default_attrs = (('AWS::IAM::Role', 'RoleName'), ('AWS::Events::Rule', 'Name'))
     for entry in default_attrs:
         if res_type == entry[0] and not props.get(entry[1]):
-            props_str = canonical_json(json_safe(props))
-            props[entry[1]] = 'cf-%s-%s' % (stack_name, md5(props_str))
+            if not resource_id:
+                resource_id = canonical_json(json_safe(props))
+                resource_id = md5(resource_id)
+            props[entry[1]] = 'cf-%s-%s' % (stack_name, resource_id)
 
 
 def apply_patches():
@@ -281,9 +282,10 @@ def apply_patches():
         try:
             if hasattr(resources_map, '_deleted'):
                 return
-            return _parse_and_create_resource(
+            result = _parse_and_create_resource(
                 logical_id, resource_json, resources_map, region_name, force_create=force_create
             )
+            return result
         except Exception as e:
             LOG.error('Unable to parse and create resource "%s": %s %s' % (logical_id, e, traceback.format_exc()))
             raise
@@ -306,6 +308,7 @@ def apply_patches():
         LOG.debug('Currently processing stack resource %s/%s: %s' % (stack_name, logical_id, updating))
         if updating:
             return None
+        CURRENTLY_UPDATING_RESOURCES[resource_hash_key] = True
 
         # parse and get final resource JSON
         resource_tuple = parsing.parse_resource_and_generate_name(logical_id, resource_json, resources_map)
@@ -314,9 +317,9 @@ def apply_patches():
         _, resource_json, resource_name = resource_tuple
 
         # add some fixes and default props which otherwise cause deployments to fail
-        add_default_resource_props(resource_json, stack_name, resource_name=resource_name)
-        for resource in resources_map._resource_json_map.values():
-            add_default_resource_props(resource, stack_name)
+        add_default_resource_props(resource_json, stack_name, resource_name=resource_name, resource_id=logical_id)
+        for res_id, resource in resources_map._resource_json_map.items():
+            add_default_resource_props(resource, stack_name, resource_id=res_id)
 
         # check if this resource already exists in the resource map
         resource = resources_map._parsed_resources.get(logical_id)
@@ -326,6 +329,7 @@ def apply_patches():
         # fix resource ARNs, make sure to convert account IDs 000000000000 to 123456789012
         resource_json_arns_fixed = clone(json_safe(convert_objs_to_ids(resource_json)))
         set_moto_account_ids(resource_json_arns_fixed)
+        template_deployer.remove_none_values(resource_json_arns_fixed)
 
         # create resource definition and store CloudFormation metadata in moto
         moto_create_error = None
@@ -390,7 +394,6 @@ def apply_patches():
                   (update, not should_be_created, is_updateable, resource_json))
 
         try:
-            CURRENTLY_UPDATING_RESOURCES[resource_hash_key] = True
             deploy_func = template_deployer.update_resource if update else template_deployer.deploy_resource
             result = deploy_func(logical_id, resource_map_new, stack_name=stack_name)
         finally:
@@ -546,6 +549,20 @@ def apply_patches():
         """
         responses.DESCRIBE_STACKS_TEMPLATE = responses.DESCRIBE_STACKS_TEMPLATE.replace(find, replace)
 
+    resource_map_diff_orig = parsing.ResourceMap.diff
+
+    def resource_map_diff(self, template, parameters=None):
+        resources_diff = resource_map_diff_orig(self, template, parameters)
+        if resources_diff['Add'] or resources_diff['Remove'] or resources_diff['Modify']:
+            return resources_diff
+
+        raise ValidationError(
+            name_or_id='',
+            message='No updates are to be performed.'
+        )
+
+    parsing.ResourceMap.diff = resource_map_diff
+
     # Patch CloudFormationBackend.update_stack method in moto
     def make_cf_update_stack(cf_backend):
         cf_update_stack_orig = cf_backend.update_stack
@@ -856,10 +873,10 @@ def apply_patches():
     @classmethod
     def Deployment_create_from_cloudformation_json(cls, resource_name, cloudformation_json, region_name):
         props = cloudformation_json['Properties']
-        name = props['StageName']
+        stage_name = props.get('StageName')
         deployment_id = props.get('Id') or short_uid()
         description = props.get('Description') or ''
-        return apigw_models.Deployment(deployment_id, name, description)
+        return apigw_models.Deployment(deployment_id, stage_name, description)
 
     @classmethod
     def Resource_create_from_cloudformation_json(cls, resource_name, cloudformation_json, region_name):
