@@ -8,7 +8,7 @@ from localstack import config
 from localstack.constants import (
     APPLICATION_AMZ_JSON_1_1, TEST_AWS_ACCOUNT_ID)
 from localstack.utils.aws import aws_stack
-from localstack.utils.common import short_uid, to_bytes
+from localstack.utils.common import short_uid, to_bytes, extract_jsonpath
 from localstack.services.infra import start_moto_server
 from localstack.services.events.scheduler import JobScheduler
 from localstack.services.awslambda.lambda_api import run_lambda
@@ -44,10 +44,28 @@ def send_event_to_firehose(event, arn):
         Record={'Data': to_bytes(json.dumps(event))})
 
 
+def filter_event_with_target_input_path(target, event):
+    input_path = target.get('InputPath')
+    if input_path:
+        event = extract_jsonpath(event, input_path)
+    return event
+
+
+def filter_event_based_on_event_format(self, rule, event):
+    rule_information = self.events_backend.describe_rule(rule)
+    if rule_information.event_pattern:
+        event_pattern = json.loads(rule_information.event_pattern)
+        for key, value in event_pattern.items():
+            if event.get(key.lower()) and event.get(key.lower()) not in value and event.get(key) != value:
+                return False
+    return True
+
+
 def process_events(event, targets):
     for target in targets:
         arn = target['Arn']
         service = arn.split(':')[2]
+        event = filter_event_with_target_input_path(target, event)
 
         if service == 'sqs':
             send_event_to_sqs(event, arn)
@@ -125,10 +143,6 @@ def apply_patches():
 
             rules = EVENT_RULES.get(event_bus, [])
 
-            targets = []
-            for rule in rules:
-                targets.extend(self.events_backend.list_targets_by_rule(rule)['Targets'])
-
             formatted_event = {
                 'version': '0',
                 'id': event_envelope['uuid'],
@@ -140,6 +154,12 @@ def apply_patches():
                 'resources': event.get('Resources', []),
                 'detail': json.loads(event.get('Detail')),
             }
+
+            targets = []
+            for rule in rules:
+                if filter_event_based_on_event_format(self, rule, formatted_event):
+                    targets.extend(self.events_backend.list_targets_by_rule(rule)['Targets'])
+
             # process event
             process_events(formatted_event, targets)
 
