@@ -2,6 +2,8 @@ import os
 import types
 import logging
 import traceback
+
+from moto.core.utils import camelcase_to_underscores
 from moto.sqs import responses as sqs_responses
 from moto.sqs.exceptions import QueueDoesNotExist
 from moto.sqs.models import Queue
@@ -9,9 +11,9 @@ from localstack import config
 from localstack.config import LOCALSTACK_HOSTNAME, TMP_FOLDER
 from localstack.utils.aws import aws_stack
 from localstack.utils.common import (
-    wait_for_port_open, save_file, short_uid, TMP_FILES, get_free_tcp_port, to_str, escape_html
+    wait_for_port_open, save_file, short_uid, TMP_FILES, get_free_tcp_port, to_str, escape_html, edge_ports_info
 )
-from localstack.services.infra import start_proxy_for_service, get_service_protocol, do_run, start_moto_server
+from localstack.services.infra import start_proxy_for_service, do_run, start_moto_server
 from localstack.services.install import INSTALL_DIR_ELASTICMQ, SQS_BACKEND_IMPL, install_elasticmq
 
 LOG = logging.getLogger(__name__)
@@ -27,7 +29,8 @@ def check_sqs(expect_shutdown=False, print_error=False):
     out = None
     try:
         # wait for port to be opened
-        wait_for_port_open(PORT_SQS_BACKEND)
+        if PORT_SQS_BACKEND:
+            wait_for_port_open(PORT_SQS_BACKEND)
         # check SQS
         out = aws_stack.connect_to_service(service_name='sqs').list_queues()
     except Exception as e:
@@ -59,6 +62,21 @@ def patch_moto():
     add_message_orig = Queue.add_message
     Queue.add_message = add_message
 
+    _set_attributes_orig = Queue._set_attributes
+
+    def _set_attributes(self, attributes, now=None):
+        _set_attributes_orig(self, attributes, now)
+
+        integer_fields = [
+            'ReceiveMessageWaitTimeSeconds'
+        ]
+
+        for key in integer_fields:
+            attribute = camelcase_to_underscores(key)
+            setattr(self, attribute, int(getattr(self, attribute, 0)))
+
+    Queue._set_attributes = _set_attributes
+
     # pass additional globals (e.g., escaping methods) to template render method
     def response_template(self, template_str, *args, **kwargs):
         template = response_template_orig(self, template_str, *args, **kwargs)
@@ -91,7 +109,9 @@ def patch_moto():
     def sqs_responses_get_queue_name(self):
         try:
             queue_url = self.querystring.get('QueueUrl')[0]
-            queue_name = queue_url.split('/')[4]
+            queue_name_data = queue_url.split('/')[4:]
+            queue_name_data = [queue_attr for queue_attr in queue_name_data if queue_attr]
+            queue_name = '/'.join(queue_name_data)
         except TypeError:
             # Fallback to reading from the URL
             queue_name = self.path.split('/')[2]
@@ -105,11 +125,9 @@ def patch_moto():
 
 
 def start_sqs_moto(port=None, asynchronous=False, update_listener=None):
-    global PORT_SQS_BACKEND
     port = port or config.PORT_SQS
-    PORT_SQS_BACKEND = get_free_tcp_port()
     patch_moto()
-    return start_moto_server('sqs', port, backend_port=PORT_SQS_BACKEND, name='SQS',
+    return start_moto_server('sqs', port, name='SQS',
         asynchronous=asynchronous, update_listener=update_listener)
 
 
@@ -141,7 +159,6 @@ def start_sqs_elasticmq(port=None, asynchronous=False, update_listener=None):
     # start process
     cmd = ('java -Dconfig.file=%s -Xmx%s -jar %s/elasticmq-server.jar' % (
         config_file, MAX_HEAP_SIZE, INSTALL_DIR_ELASTICMQ))
-    print('Starting mock SQS service in %s ports %s (recommended) and %s (deprecated)...' % (
-        get_service_protocol(), config.EDGE_PORT, port))
+    print('Starting mock SQS service on %s ...' % edge_ports_info())
     start_proxy_for_service('sqs', port, PORT_SQS_BACKEND, update_listener)
     return do_run(cmd, asynchronous)
