@@ -3,10 +3,10 @@ import json
 import base64
 from six.moves.urllib.parse import quote_plus, unquote_plus
 from localstack import config
-from localstack.utils.common import recurse_object
+from localstack.utils.common import recurse_object, extract_jsonpath
 
 
-class VelocityInput:
+class VelocityInput(object):
     """Simple class to mimick the behavior of variable '$input' in AWS API Gateway integration velocity templates.
     See: http://docs.aws.amazon.com/apigateway/latest/developerguide/api-gateway-mapping-template-reference.html"""
 
@@ -14,21 +14,20 @@ class VelocityInput:
         self.value = value
 
     def path(self, path):
-        from jsonpath_rw import parse
         value = self.value if isinstance(self.value, dict) else json.loads(self.value)
-        jsonpath_expr = parse(path)
-        result = [match.value for match in jsonpath_expr.find(value)]
-        result = result[0] if len(result) == 1 else result
-        return result
+        return extract_jsonpath(value, path)
 
     def json(self, path):
         return json.dumps(self.path(path))
 
+    def __getattr__(self, name):
+        return self.value.get(name)
+
     def __repr__(self):
         return '$input'
 
 
-class VelocityUtil:
+class VelocityUtil(object):
     """Simple class to mimick the behavior of variable '$util' in AWS API Gateway integration velocity templates.
     See: http://docs.aws.amazon.com/apigateway/latest/developerguide/api-gateway-mapping-template-reference.html"""
 
