@@ -1,6 +1,7 @@
 import os
 import re
 import sys
+import json
 import time
 import select
 import pkgutil
@@ -15,6 +16,7 @@ import pip as pip_mod
 from datetime import datetime
 from concurrent.futures._base import Future
 from localstack import constants, config
+from localstack.utils.analytics.profiler import log_duration
 
 # set up logger
 LOG = logging.getLogger(os.path.basename(__file__))
@@ -43,14 +45,16 @@ PLUGIN_SCOPE_COMMANDS = 'commands'
 
 # maps from API names to list of other API names that they depend on
 API_DEPENDENCIES = {
+    'dynamodb': ['dynamodbstreams'],
     'dynamodbstreams': ['kinesis'],
     'es': ['elasticsearch'],
-    'lambda': ['logs'],
-    'kinesis': ['dynamodb']
+    'lambda': ['logs', 'cloudwatch'],
+    'kinesis': ['dynamodb'],
+    'firehose': ['kinesis']
 }
 # composites define an abstract name like "serverless" that maps to a set of services
 API_COMPOSITES = {
-    'serverless': ['cloudformation', 'iam', 'sts', 'lambda', 'dynamodb', 'apigateway', 's3'],
+    'serverless': ['cloudformation', 'cloudwatch', 'iam', 'sts', 'lambda', 'dynamodb', 'apigateway', 's3'],
     'cognito': ['cognito-idp', 'cognito-identity']
 }
 
@@ -98,6 +102,7 @@ def run_pip_main(args):
     return pip._internal.main.main(args)
 
 
+@log_duration()
 def load_plugin_from_path(file_path, scope=None):
     if os.path.exists(file_path):
         module = re.sub(r'(^|.+/)([^/]+)/plugins.py', r'\2', file_path)
@@ -122,13 +127,21 @@ def load_plugin_from_path(file_path, scope=None):
                 LOG.warning('Unable to load plugins from file %s: %s' % (file_path, e))
 
 
+def should_load_module(module, scope):
+    if module == 'localstack_ext' and not os.environ.get('LOCALSTACK_API_KEY'):
+        return False
+    return True
+
+
+@log_duration()
 def load_plugins(scope=None):
     scope = scope or PLUGIN_SCOPE_SERVICES
     if PLUGINS_LOADED.get(scope):
         return PLUGINS_LOADED[scope]
 
     t1 = now_utc()
-    setup_logging()
+    log_level = logging.WARNING if scope == PLUGIN_SCOPE_COMMANDS else None
+    setup_logging(log_level=log_level)
 
     loaded_files = []
     result = []
@@ -138,6 +151,8 @@ def load_plugins(scope=None):
     search_modules = PLUGIN_MODULES
 
     for module in search_modules:
+        if not should_load_module(module, scope):
+            continue
         file_path = None
         if isinstance(module, six.string_types):
             loader = pkgutil.get_loader(module)
@@ -172,6 +187,25 @@ def docker_container_running(container_name):
     return container_name in container_names
 
 
+def get_docker_image_details(image_name=None):
+    image_name = image_name or get_docker_image_to_start()
+    try:
+        result = run('%s inspect %s' % (config.DOCKER_CMD, image_name), print_error=False)
+        result = json.loads(to_str(result))
+        assert len(result)
+    except Exception:
+        return {}
+    if len(result) > 1:
+        LOG.warning('Found multiple images (%s) named "%s"' % (len(result), image_name))
+    result = result[0]
+    result = {
+        'id': result['Id'].replace('sha256:', '')[:12],
+        'tag': (result.get('RepoTags') or ['latest'])[0].split(':')[-1],
+        'created': result['Created'].split('.')[0]
+    }
+    return result
+
+
 def get_docker_container_names():
     cmd = "%s ps --format '{{.Names}}'" % config.DOCKER_CMD
     try:
@@ -192,15 +226,38 @@ def get_main_container_ip():
 
 def get_main_container_name():
     cmd = "%s inspect -f '{{ .Name }}' %s" % (config.DOCKER_CMD, config.HOSTNAME)
-    return run(cmd).strip().lstrip('/')
+    try:
+        return run(cmd, print_error=False).strip().lstrip('/')
+    except Exception:
+        return config.MAIN_CONTAINER_NAME
 
 
-def setup_logging():
+def get_server_version():
+    docker_cmd = config.DOCKER_CMD
+    try:
+        # try to extract from existing running container
+        container_name = get_main_container_name()
+        version = run('%s exec -it %s bin/localstack --version' % (docker_cmd, container_name), print_error=False)
+        version = version.strip().split('\n')[-1]
+        return version
+    except Exception:
+        try:
+            # try to extract by starting a new container
+            img_name = get_docker_image_to_start()
+            version = run('%s run --entrypoint= -it %s bin/localstack --version' % (docker_cmd, img_name))
+            version = version.strip().split('\n')[-1]
+            return version
+        except Exception:
+            # fall back to default constant
+            return constants.VERSION
+
+
+def setup_logging(log_level=None):
     # determine and set log level
     if PLUGINS_LOADED.get('_logging_'):
         return
     PLUGINS_LOADED['_logging_'] = True
-    log_level = logging.DEBUG if is_debug() else logging.INFO
+    log_level = log_level or (logging.DEBUG if is_debug() else logging.INFO)
     logging.basicConfig(level=log_level, format=LOG_FORMAT, datefmt=LOG_DATE_FORMAT)
 
     # set up werkzeug logger
@@ -307,7 +364,7 @@ class PortMappings(object):
             for i in range(port[1] - port[0] + 1):
                 self.add(port[0] + i, mapped[0] + i)
             return
-        if int(port) <= 0:
+        if port is None or int(port) <= 0:
             raise Exception('Unable to add mapping for invalid port: %s' % port)
         if self.contains(port):
             return
@@ -520,6 +577,7 @@ class FuncThread(threading.Thread):
         try:
             result = self.func(self.params)
         except Exception as e:
+            result = e
             if not self.quiet:
                 LOG.warning('Thread run method %s(%s) failed: %s %s' %
                     (self.func, self.params, e, traceback.format_exc()))
