@@ -229,6 +229,12 @@ def iam_create_policy_params(params, **kwargs):
     return result
 
 
+def lambda_permission_params(params, **kwargs):
+    result = select_parameters('FunctionName', 'Action', 'Principal')(params, **kwargs)
+    result['StatementId'] = common.short_uid()
+    return result
+
+
 def get_ddb_provisioned_throughput(params, **kwargs):
     args = params.get('ProvisionedThroughput')
     if args:
@@ -305,9 +311,10 @@ RESOURCE_TO_FUNCTION = {
             'function': 'create_queue',
             'parameters': {
                 'QueueName': ['QueueName', PLACEHOLDER_RESOURCE_NAME],
-                'Attributes': params_select_attributes('ContentBasedDeduplication', 'DelaySeconds', 'FifoQueue',
-                                                       'MaximumMessageSize', 'MessageRetentionPeriod',
-                                                       'VisibilityTimeout', 'RedrivePolicy'),
+                'Attributes': params_select_attributes(
+                    'ContentBasedDeduplication', 'DelaySeconds', 'FifoQueue', 'MaximumMessageSize',
+                    'MessageRetentionPeriod', 'VisibilityTimeout', 'RedrivePolicy', 'ReceiveMessageWaitTimeSeconds'
+                ),
                 'tags': params_list_to_dict('Tags')
             }
         },
@@ -426,25 +433,20 @@ RESOURCE_TO_FUNCTION = {
     'Lambda::Version': {
         'create': {
             'function': 'publish_version',
-            'parameters': {
-                'FunctionName': 'FunctionName',
-                'CodeSha256': 'CodeSha256',
-                'Description': 'Description'
-            }
+            'parameters': select_parameters('FunctionName', 'CodeSha256', 'Description')
+        }
+    },
+    'Lambda::Permission': {
+        'create': {
+            'function': 'add_permission',
+            'parameters': lambda_permission_params
         }
     },
-    'Lambda::Permission': {},
     'Lambda::EventSourceMapping': {
         'create': {
             'function': 'create_event_source_mapping',
-            'parameters': {
-                'FunctionName': 'FunctionName',
-                'EventSourceArn': 'EventSourceArn',
-                'StartingPosition': 'StartingPosition',
-                'Enabled': 'Enabled',
-                'BatchSize': 'BatchSize',
-                'StartingPositionTimestamp': 'StartingPositionTimestamp'
-            }
+            'parameters': select_parameters('FunctionName', 'EventSourceArn', 'Enabled',
+                'StartingPosition', 'BatchSize', 'StartingPositionTimestamp')
         }
     },
     'DynamoDB::Table': {
@@ -1071,6 +1073,7 @@ def update_resource(resource_id, resources, stack_name):
         return
     LOG.info('Updating resource %s of type %s' % (resource_id, resource_type))
     props = resource['Properties']
+
     if resource_type == 'Lambda::Function':
         client = aws_stack.connect_to_service('lambda')
         keys = ('FunctionName', 'Role', 'Handler', 'Description', 'Timeout', 'MemorySize', 'Environment', 'Runtime')
@@ -1078,7 +1081,12 @@ def update_resource(resource_id, resources, stack_name):
         update_props = resolve_refs_recursively(stack_name, update_props, resources)
         if 'Code' in props:
             client.update_function_code(FunctionName=props['FunctionName'], **props['Code'])
+        if 'Environment' in update_props:
+            environment_variables = update_props['Environment'].get('Variables', {})
+            update_props['Environment']['Variables'] = {k: str(v) for k, v in environment_variables.items()}
+
         return client.update_function_configuration(**update_props)
+
     if resource_type == 'ApiGateway::Method':
         client = aws_stack.connect_to_service('apigateway')
         integration = props.get('Integration')
@@ -1095,6 +1103,7 @@ def update_resource(resource_id, resources, stack_name):
             kwargs['uri'] = integration.get('Uri')
             return client.put_integration(**kwargs)
         kwargs['authorizationType'] = props.get('AuthorizationType')
+
         return client.put_method(**kwargs)
 
 
@@ -1200,6 +1209,21 @@ def execute_resource_action(resource_id, resources, stack_name, action_name):
     return (results or [None])[0]
 
 
+def fix_resource_props_for_sdk_deployment(resource_type, resource_props):
+    if resource_type == 'Lambda::Function':
+        # Properties will be validated by botocore before sending request to AWS
+        # botocore/data/lambda/2015-03-31/service-2.json:1161 (EnvironmentVariableValue)
+        # https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-lambda-function-environment.html
+        if 'Environment' in resource_props:
+            environment_variables = resource_props['Environment'].get('Variables', {})
+            resource_props['Environment']['Variables'] = {k: str(v) for k, v in environment_variables.items()}
+
+    if resource_type == 'SQS::Queue':
+        # https://github.com/localstack/localstack/issues/3004
+        if 'ReceiveMessageWaitTimeSeconds' in resource_props:
+            resource_props['ReceiveMessageWaitTimeSeconds'] = int(resource_props['ReceiveMessageWaitTimeSeconds'])
+
+
 def configure_resource_via_sdk(resource_id, resources, resource_type, func_details, stack_name):
     resource = resources[resource_id]
     client = get_client(resource, func_details)
@@ -1211,13 +1235,7 @@ def configure_resource_via_sdk(resource_id, resources, resource_type, func_detai
     resource_props = resource['Properties']
 
     # Validate props for each resource type
-    if resource_type == 'Lambda::Function':
-        # Properties will be validated by botocore before sending request to AWS
-        # botocore/data/lambda/2015-03-31/service-2.json:1161 (EnvironmentVariableValue)
-        # https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-lambda-function-environment.html
-        if 'Environment' in resource_props:
-            environment_variables = resource_props['Environment'].get('Variables', {})
-            resource_props['Environment']['Variables'] = {k: str(v) for k, v in environment_variables.items()}
+    fix_resource_props_for_sdk_deployment(resource_type, resource_props)
 
     if callable(params):
         params = params(resource_props, stack_name=stack_name, resources=resources)
