@@ -1,8 +1,6 @@
 import io
 import os
 import re
-import pwd
-import grp
 import sys
 import json
 import uuid
@@ -18,16 +16,16 @@ import zipfile
 import binascii
 import calendar
 import tempfile
+import functools
 import threading
 import subprocess
 import six
 import shutil
 import requests
 import dns.resolver
-import functools
 from io import BytesIO
-from contextlib import closing
 from datetime import datetime, date
+from contextlib import closing
 from six import with_metaclass
 from six.moves import cStringIO as StringIO
 from six.moves.urllib.parse import urlparse, parse_qs
@@ -99,14 +97,15 @@ class ShellCommandThread(FuncThread):
     """ Helper class to run a shell command in a background thread. """
 
     def __init__(self, cmd, params={}, outfile=None, env_vars={}, stdin=False,
-            quiet=True, inherit_cwd=False, inherit_env=True):
+            quiet=True, inherit_cwd=False, inherit_env=True, log_listener=None):
         self.cmd = cmd
         self.process = None
-        self.outfile = outfile or os.devnull
+        self.outfile = outfile
         self.stdin = stdin
         self.env_vars = env_vars
         self.inherit_cwd = inherit_cwd
         self.inherit_env = inherit_env
+        self.log_listener = log_listener
         FuncThread.__init__(self, self.run_cmd, params, quiet=quiet)
 
     def run_cmd(self, params):
@@ -119,11 +118,14 @@ class ShellCommandThread(FuncThread):
             """ Return True if this line should be filtered, i.e., not printed """
             return '(Press CTRL+C to quit)' in line
 
+        outfile = self.outfile or os.devnull
+        if self.log_listener and outfile == os.devnull:
+            outfile = subprocess.PIPE
         try:
-            self.process = run(self.cmd, asynchronous=True, stdin=self.stdin, outfile=self.outfile,
+            self.process = run(self.cmd, asynchronous=True, stdin=self.stdin, outfile=outfile,
                 env_vars=self.env_vars, inherit_cwd=self.inherit_cwd, inherit_env=self.inherit_env)
-            if self.outfile:
-                if self.outfile == subprocess.PIPE:
+            if outfile:
+                if outfile == subprocess.PIPE:
                     # get stdout/stderr from child process and write to parent output
                     streams = ((self.process.stdout, sys.stdout), (self.process.stderr, sys.stderr))
                     for instream, outstream in streams:
@@ -137,8 +139,11 @@ class ShellCommandThread(FuncThread):
                             line = convert_line(line)
                             if filter_line(line):
                                 continue
-                            outstream.write(line)
-                            outstream.flush()
+                            if self.log_listener:
+                                self.log_listener(line, stream=instream)
+                            if self.outfile not in [None, os.devnull]:
+                                outstream.write(line)
+                                outstream.flush()
                 self.process.wait()
             else:
                 self.process.communicate()
@@ -485,6 +490,10 @@ def timestamp_millis(time=None):
     return microsecond_time[:-4] + microsecond_time[-1]
 
 
+def epoch_timestamp():
+    return time.time()
+
+
 def retry(function, retries=3, sleep=1, sleep_before=0, **kwargs):
     raise_error = None
     if sleep_before > 0:
@@ -598,6 +607,9 @@ def ensure_readable(file_path, default_perms=None):
 
 def chown_r(path, user):
     """ Recursive chown """
+    # keep these imports here for Windows compatibility
+    import pwd
+    import grp
     uid = pwd.getpwnam(user).pw_uid
     gid = grp.getgrnam(user).gr_gid
     os.chown(path, uid, gid)
@@ -643,9 +655,8 @@ def rm_rf(path):
 def cp_r(src, dst):
     """Recursively copies file/directory"""
     if os.path.isfile(src):
-        shutil.copy(src, dst)
-    else:
-        shutil.copytree(src, dst)
+        return shutil.copy(src, dst)
+    return shutil.copytree(src, dst)
 
 
 def download(url, path, verify_ssl=True):
@@ -775,10 +786,21 @@ def canonical_json(obj):
     return json.dumps(obj, sort_keys=True)
 
 
+def extract_jsonpath(value, path):
+    from jsonpath_rw import parse
+    jsonpath_expr = parse(path)
+    result = [match.value for match in jsonpath_expr.find(value)]
+    result = result[0] if len(result) == 1 else result
+    return result
+
+
 def save_file(file, content, append=False):
     mode = 'a' if append else 'w+'
     if not isinstance(content, six.string_types):
         mode = mode + 'b'
+    # make sure that the parent dir exsits
+    mkdir(os.path.dirname(file))
+    # store file contents
     with open(file, mode) as f:
         f.write(content)
         f.flush()
@@ -1117,7 +1139,6 @@ def run_cmd_safe(**kwargs):
 
 
 def run(cmd, cache_duration_secs=0, **kwargs):
-
     def do_run(cmd):
         return bootstrap.run(cmd, **kwargs)
 
@@ -1190,7 +1211,6 @@ class safe_requests(with_metaclass(_RequestsSafe)):
 
 
 def make_http_request(url, data=None, headers=None, method='GET'):
-
     if is_string(method):
         method = requests.__dict__[method.lower()]
 
@@ -1251,5 +1271,10 @@ def isoformat_milliseconds(t):
         return t.isoformat()[:-3]
 
 
+def _replace(response, pattern, replacement):
+    content = to_str(response.content)
+    response._content = re.sub(pattern, replacement, content)
+
+
 # Code that requires util functions from above
 CACHE_FILE_PATTERN = CACHE_FILE_PATTERN.replace('_random_dir_', short_uid())
